`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:26:06 02/12/2017 
// Design Name: 
// Module Name:    i2c_interface
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module i2c_interface		#(parameter RETRY_CNT = 10'h3F0) 

							(input clk_slow, input rst, (* MARK_DEBUG = "true" *)input [7:0] slave_addr, (* MARK_DEBUG = "true" *) input [7:0] reg_addr, (* MARK_DEBUG = "true" *) input [7:0] data_in, (* MARK_DEBUG = "true" *) input WR_fast, (* MARK_DEBUG = "true" *) input RD_fast,
							(* MARK_DEBUG = "true" *) input retry, input [1:0] rd_bytes, (* MARK_DEBUG = "true" *) input no_data,
								
							inout SDA, inout SCL, 
								
							output reg [31:0] data_out, (* MARK_DEBUG = "true" *) output reg valid, (* MARK_DEBUG = "true" *) output reg ack, (* MARK_DEBUG = "true" *) output reg complete);

								
(* FSM_ENCODING="USER" *) (* MARK_DEBUG = "true" *) reg [4:0] STATE;
(* FSM_ENCODING="USER" *) (* MARK_DEBUG = "true" *) reg [1:0] BIT;
(* MARK_DEBUG = "true" *) reg [2:0] bit_counter;
(* MARK_DEBUG = "true" *) reg [9:0] retry_cnt;
(* MARK_DEBUG = "true" *) reg sda_en_reg, scl_en_reg;
(* MARK_DEBUG = "true" *) reg [7:0] sr_slave_addr, sr_reg_addr, sr_data_byte, sack_timeout;
(* MARK_DEBUG = "true" *) reg [31:0] sr_read_data;

(* MARK_DEBUG = "true" *) reg [1:0] scl_in_reg, rd_bytes_reg;
(* MARK_DEBUG = "true" *) reg sda_in_reg, RD_WR_reg, RD, WR;

reg [1:0] WR_slow, RD_slow;
reg write_enable, crc_rst;
wire rst_internal;
(* MARK_DEBUG = "true" *) wire [7:0] crc_chk_byte;

(* MARK_DEBUG = "true" *) wire sda_in, scl_in;

//clk_fast brings in the WR/RD signals, so they must be brought only once into the slow domain
always@(posedge clk_slow)
begin
	WR_slow <= {WR_slow[0], WR_fast};
	RD_slow <= {RD_slow[0], RD_fast};
	WR <= (WR_slow == 2'b01) ? 1'b1 : 1'b0;
	RD <= (RD_slow == 2'b01) ? 1'b1 : 1'b0;
end


// For the Fast mode operation, resulting SCL must be 400 KHz or less.  Since SCL is 1/3 frequency of clk_slow, clk_slow must be 1200 KHz or less
assign SCL = scl_en_reg ? 1'bz : 1'b0;
assign scl_in = SCL;

assign SDA = sda_en_reg ? 1'bz : 1'b0;
assign sda_in = SDA;


localparam IDLE 		= 5'h00;
localparam START		= 5'h01;
localparam ADDR		    = 5'h02;
localparam ACK_0		= 5'h03;
localparam ACK_1		= 5'h04;
localparam ACK_2		= 5'h05;
localparam SADDR		= 5'h06;
localparam SACK_0		= 5'h07;
localparam SACK_1		= 5'h08;
localparam SACK_2		= 5'h09;
localparam DATA		    = 5'h0A;
localparam DACK_0		= 5'h0B;
localparam DACK_1		= 5'h0C;
localparam DACK_2		= 5'h0D;
localparam STOP_0		= 5'h0E;
localparam STOP_1		= 5'h0F;

localparam RESTART_0    = 5'h10;
localparam RESTART_1    = 5'h11;
localparam RESTART_2    = 5'h12;
localparam RADDR	  	= 5'h13;
localparam RACK_0		= 5'h14;
localparam RACK_1		= 5'h15;
localparam RACK_2		= 5'h16;
localparam READ		    = 5'h17;
localparam WACK_0		= 5'h18;
localparam WACK_1		= 5'h19;
localparam WACK_2 	    = 5'h1A;
localparam NAK_0		= 5'h1E;
localparam NAK_1		= 5'h1F;

localparam ZERO		    = 2'b00;
localparam ONE			= 2'b01;
localparam TWO	 		= 2'b10;
localparam THREE		= 2'b11;

localparam GENERAL_CALL = 8'h00;

initial
begin
sda_en_reg <= 1'b1;
scl_en_reg <= 1'b1;
STATE <= IDLE;
BIT <= ZERO;
valid <= 1'b0;
complete <= 1'b0;
bit_counter <= 3'h0;
ack <= 1'b0;
data_out <= 32'h00000000;
sr_slave_addr <= 8'h00;
sr_reg_addr <= 8'h00;
sr_read_data <= 32'h00000000;
sr_data_byte <= 8'h00;
RD_WR_reg <= 1'b0;
sack_timeout <= 8'h00;
retry_cnt <= RETRY_CNT;
crc_rst <= 1'b1;
end


always@(posedge clk_slow)															// synchronous to "clk_slow"  The SCL signal must be 400 KHz or less.
																					// Since SCL is 1/3 frequency of clk_slow, clk_slow must be 1.200 MHz or less
begin
		if (rst)																	// synchronous, active high reset
			begin
			sda_en_reg <= 1'b1;
			scl_en_reg <= 1'b1;
			STATE <= IDLE;
			BIT <= ZERO;
			valid <= 1'b0;
			complete <= 1'b0;
			bit_counter <= 3'h0;
			ack <= 1'b0;
			RD_WR_reg <= 1'b0;
			crc_rst <= 1'b1;
			end
			else
				begin
									
				scl_in_reg <= {scl_in_reg [0], scl_in};								// scl pin sampled on every clock, stored in two bit register to show current and previous value
				sda_in_reg <= sda_in;												// sda pin sampled on every clock
				
				case (STATE)
					
					IDLE		:	begin
									if (WR | RD)									// if a WR or RD is commanded, start the state machine
										begin
										STATE <= START;
										sda_en_reg <= 1'b0;							// this will drive SDA low, SCL remains high (I2C start condition)
										sr_slave_addr <= slave_addr;				// load the slave address, byte address, and data byte into registers
										sr_reg_addr <= reg_addr;
										sr_data_byte <= data_in;						
										RD_WR_reg <= !WR;							// capture the WR or RD command in the RD_WR_reg register
										ack <= 1'b1;								// acknowledge the WR or RD command
										crc_rst <= 1'b0;							// remove the rst to CRC block
										end
											else
												begin
												sda_en_reg <= 1'b1;					// if a WR or RD is not commanded, then keep SDA and SCL tristated, and keep the CRC block in reset
												scl_en_reg <= 1'b1;
												crc_rst <= 1'b1;
												end
									complete <= 1'b0;								// if a WR or RD is not commanded, then hold the complete and valid signals low
									valid <= 1'b0;
									if (valid && (crc_chk_byte == sr_read_data [7:0])) data_out <= sr_read_data;
																					// if the I2C transaction was valid, and the CRC check byte is correct, present the data to the sr_read_data output 
									end
									
					START		:	begin
									STATE <= ADDR;									// go the address state 
									scl_en_reg <= 1'b0;								// drive SCL low = start
									ack <= 1'b0;									// remove the acknowledge 
									end
									
					ADDR		:	begin					
									case (BIT)
									
									ZERO   :	begin
												BIT <= ONE;
												sr_slave_addr <= {sr_slave_addr [6:0], sr_slave_addr [7]};
												sda_en_reg <= sr_slave_addr [7];	// drives SDA low or releases to float high depending on bit
												end
															
									ONE    :	begin
												BIT <= TWO;
												scl_en_reg <= 1'b1;					// release SCL to float high
												end
										
															
									TWO	   :	begin
												BIT <= ZERO;
												if (bit_counter == 3'h7) STATE <= ACK_0;	// if the address bit counter is 3'h7, go to the acknowledge state
												scl_en_reg <= 1'b0;							// drive SCL back low, go to next bit
												bit_counter <= bit_counter + 1'b1;
												end
															
									default	:	BIT <= ZERO;						// start the bit counter at zero															
															
									endcase
									end
									
					ACK_0		:	begin
									STATE <= ACK_1;
									sda_en_reg <= 1'b1;								// let the SDA float HIGH (not acknowledge) or be externally held low (acknowledge)
									end
									
					ACK_1		:	begin	
									STATE <= ACK_2;
									scl_en_reg <= 1'b1;								// release SCL to float high
									end
									
					ACK_2		:	begin											// drive the SCL low
									scl_en_reg <= 1'b0;
									if ((scl_in_reg == 2'b01) && (sr_slave_addr == GENERAL_CALL)) STATE <= DATA;		// if this is broadcast, go to DATA
										else if ((scl_in_reg == 2'b01) && (!sda_in_reg)) STATE <= SADDR; 				// if there was a slave acknowledge, go to slave address state
											else if (scl_in_reg == 2'b01) STATE <= NAK_0;								// if there was a not acknowledge, goto not acknowledge state
									end
									
					SADDR		:	begin					
									case (BIT)
									
									ZERO	:	begin
												BIT <= ONE;
												sr_reg_addr <= {sr_reg_addr [6:0], sr_reg_addr [7]};
												sda_en_reg <= sr_reg_addr [7];		// drives SDA low or releases to float high depending on bit
												end
															
									ONE	   :	begin
												BIT <= TWO;
												scl_en_reg <= 1'b1;					// release SCL to float high
												end
										
															
									TWO	   :	begin
												BIT <= ZERO;
												scl_en_reg <= 1'b0;					// drive SCL back low, go to next bit
												bit_counter <= bit_counter + 1'b1;
												if (bit_counter == 3'h7) STATE <= SACK_0; // if the address bit counter is 3'h7, go to the sub addr acknowledge state
												end
															
									default	:	BIT <= ZERO;						// start the bit counter at zero	
																				
									endcase
									end
									
					SACK_0	    :	begin
									STATE <= SACK_1;
									sda_en_reg <= 1'b1;								// let the SDA float HIGH (not acknowledge) or be externally held low (acknowledge)
									end
									
					SACK_1	    :	begin	
									STATE <= SACK_2;
									scl_en_reg <= 1'b1;								// release SCL to float high
									sack_timeout <= 8'h00;							// reset the acknowledge timeout  
																					// in the case of clock stretching, wait a certain time for the slave to release the SCL high
									end			
					
					SACK_2	    :	begin														
									sack_timeout <= sack_timeout + 1'b1;			// increment the timeout counter waiting for the clock edge
									if ((scl_in_reg == 2'b01) && (!sda_in_reg))
										begin
										if (no_data) STATE <= STOP_0;
											else STATE <= (RD_WR_reg) ? RESTART_0 : DATA; // if slave acknowledges, and there will be data, go to data or restart state
										retry_cnt <= (retry)? RETRY_CNT : 10'h000;		  // if slave doesn't stretch the clock, there retries will be used, reset the retry counter
										scl_en_reg <= 1'b0;								  // drive the SCL low
										end
										else if (scl_in_reg == 2'b01)
											begin
											STATE <= NAK_0;								  // if there slave did not acknowledge, go to the not acknowledge state
											scl_en_reg <= 1'b0;							  // drive the SCL low
											end
											else if (&sack_timeout)
												begin
												STATE <= NAK_0;							  // in the case of no clock edge, also go to the not acknowledge state	
												scl_en_reg <= 1'b0;						  // drive the SCL low
												end
									end
					
					RESTART_0  :	begin
									STATE <= RESTART_1;
									scl_en_reg <= 1'b1;									  // release SCL to float high for the coming restart
									end
									
					RESTART_1  :	begin
									STATE <= RESTART_2;
									sda_en_reg <= 1'b0;									  // drive SDA low for the restart
									sr_slave_addr <= {sr_slave_addr [7:1], RD_WR_reg};    // load the register address
									end
									
					RESTART_2  :	begin
									scl_en_reg <= 1'b0;									  // drive SCL low for completion of restart
									STATE <= RADDR;
									end
									
					RADDR	   :	begin					
									case (BIT)
									
									ZERO	:	begin
												BIT <= ONE;
												sr_slave_addr <= {sr_slave_addr [6:0], sr_slave_addr [7]};
												sda_en_reg <= sr_slave_addr [7];				// drives SDA low or releases to float high depending on bit
												end
															
									ONE	   :	begin
												BIT <= TWO;
												scl_en_reg <= 1'b1;								// release SCL to float high	
												end
		
									TWO	   :	begin
												BIT <= ZERO;
												scl_en_reg <= 1'b0;								// drive SCL back low, go to next bit
												bit_counter <= bit_counter + 1'b1;
												if (bit_counter == 3'h7) STATE <= RACK_0;       // if the address bit counter is 3'h7, go to the register addr acknowledge state
												end
															
									default	:	BIT <= ZERO;
																				
									endcase
									end								
									
					RACK_0	    :	begin
									STATE <= RACK_1;
									sda_en_reg <= 1'b1;											// let the SDA float HIGH (not acknowledge) or be externally held low (acknowledge)
									end
									
					RACK_1	    :	begin																
									STATE <= RACK_2;
									scl_en_reg <= 1'b1;											// release SCL to float high
									end
									
					RACK_2	    :	begin																
									scl_en_reg <= 1'b0;											// drive the SCL LOW
									if ((scl_in_reg == 2'b01) && (!sda_in_reg))			        // if there was a slave acknowledge, go to read state
										begin
										STATE <= READ;												
										rd_bytes_reg <= rd_bytes;								// load the number of bytes (0-3) that will be read out
										end
										else if (scl_in_reg == 2'b01)							// if there was no acknowledge, start the retry counter
											begin
											if (retry_cnt > 10'h000) 
												begin
												STATE <= RESTART_0;								// go to the restart state and try again
												retry_cnt <= retry_cnt - 1'b1;				    // decrement the retry counter				
												end
												else STATE <= NAK_0;								// if there slave did not acknowledge, go to the not acknowledge state
											end
									end
									
					READ		:	begin
									if (scl_in_reg == 2'b01)									// do the following operations on each positive edge of the SCL
										begin
										sr_read_data <= {sr_read_data [30:0], sda_in_reg};      // load the sr_read_data with the data bit on the SDA pin
										end
										
										case (BIT)
										
										ZERO	:	begin
													BIT <= ONE;
													scl_en_reg <= 1'b1;							// release SCL to float high
													end
																
										ONE	    :	begin
													BIT <= TWO;
													scl_en_reg <= 1'b0;							// drive SCL low
													end
																
										TWO	    :	begin
													if (scl_in_reg [0])							// implement the clock stretching check
													begin
														bit_counter <= bit_counter + 1'b1;
														if (bit_counter == 3'h7)
															begin
															STATE <= WACK_0;
															sda_en_reg <= (rd_bytes_reg > 2'b00) ? 1'b0 : 1'b1;		// if it's time to WNACK (on the last read byte), put the SDA HIGH
															valid <= 1'b0;
															end
													end
																	
													BIT <= ZERO;
																	
													end
																
										default	:	BIT <= ZERO;
																
										endcase

									end
									
					WACK_0	    :	begin
									STATE <= WACK_1;
									scl_en_reg <= 1'b1;											// release SCL to float high
									end
									
					WACK_1	    :	begin
									scl_en_reg <= 1'b0;											// drive SCL low
									if (rd_bytes_reg > 2'b00)									// if there are still read bytes, go WACK_2 and then READ
										begin
										STATE <= WACK_2;
										rd_bytes_reg <= rd_bytes_reg - 1'b1;
										end
										else STATE <= STOP_0;									// if there are no more read bytes, go to STOP_0
																		
									end
									
					WACK_2	    :	begin
									STATE <= READ;
									sda_en_reg <= 1'b1;
									end
									
					DATA		:	begin
					
									case (BIT)
									
										ZERO		:		begin
															sr_data_byte <= {sr_data_byte [6:0], sr_data_byte [7]};		// can be byte address, could be data depending on the write_data
															sda_en_reg <= sr_data_byte [7];										// drives SDA low or high depending on bit
															BIT <= ONE;	
															end
															
										ONE		     :		begin
															scl_en_reg <= 1'b1;								// let the SCK be pulled HIGH
															BIT <= TWO;
															end
															
										TWO		     :		begin
															scl_en_reg <= 1'b0;								// drive SCL back low, go to next bit
															bit_counter <= bit_counter + 1'b1;
															BIT <= ZERO;
															if (bit_counter == 3'h7) STATE <= DACK_0;
															end
															
										default	:		BIT <= ZERO;
					
									endcase

									end
									
					DACK_0	    :	begin
									sda_en_reg <= 1'b1;											// let go the SDA to be pulled HIGH (NAK) or held LOW (Slave ACK)
									STATE <= DACK_1;
									end
									
					DACK_1	    :	begin														// let the SCK be pulled HIGH
									scl_en_reg <= 1'b1;
									STATE <= DACK_2;
									end
									
					DACK_2	    :	begin														// drive the SCK LOW
									scl_en_reg <= 1'b0;
									if ((scl_in_reg == 2'b01) && (!sda_in_reg)) STATE <= STOP_0;	// if there was Slave ACK, go to STOP_0
										else if  ((scl_in_reg == 2'b01) && (sr_slave_addr == GENERAL_CALL)) STATE <= STOP_0;		// if this is broadcast, go to STOP_0
											else if (scl_in_reg == 2'b01) STATE <= NAK_0;		// if there was a NAK, goto NAK
									end
									
					STOP_0	    :	begin
									sda_en_reg <= 1'b0;											// keep SDA low in preparation for STOP_1
									STATE <= STOP_1;
									end
									
					STOP_1	    :	begin
									scl_en_reg <= 1'b1;											// let SCL go high in preparation for STOP condition completed in IDLE
									STATE <= IDLE;
									complete <= 1'b1;
									valid <= 1'b1;
									//data_out <= sr_read_data;
									end
									
					NAK_0		:	begin
									sda_en_reg <= 1'b0;											// keep SDA low in preparation for NAK_1
									STATE <= NAK_1;
									valid <= 1'b0;
									end
									
					NAK_1		:	begin
									scl_en_reg <= 1'b1;											// let SCL go high in preparation for STOP condition completed in IDLE
									STATE <= IDLE;
									complete <= 1'b1;
									end
									
								
					default	   :	begin
									STATE <= IDLE;
									BIT <= ZERO;
									bit_counter <= 3'h0;
									sda_en_reg <= 1'b1;
									scl_en_reg <= 1'b1;
									valid <= 1'b0;
									complete <= 1'b0;
									ack <= 1'b0;
									end
				

				endcase

				end
end


crc si7020_crc (.data_in(sr_read_data [23:8]), .crc_en(valid), .crc_out(crc_chk_byte), .rst(crc_rst), .clk(clk_slow));


endmodule

//-----------------------------------------------------------------------------
// CRC CALC ----------  crc[7:0]=1+x^4+x^5+x^8;
//-----------------------------------------------------------------------------
module crc (input [15:0] data_in, input crc_en, output [7:0] crc_out, input rst, input clk);

  reg [7:0] lfsr_q,lfsr_c;

  assign crc_out = lfsr_q;

  always @(*) begin
    lfsr_c[0] = lfsr_q[1] ^ lfsr_q[2] ^ lfsr_q[3] ^ lfsr_q[6] ^ lfsr_q[7] ^ data_in[0] ^ data_in[3] ^ data_in[4] ^ data_in[6] ^ data_in[9] ^ data_in[10] ^ data_in[11] ^ data_in[14] ^ data_in[15];
    lfsr_c[1] = lfsr_q[2] ^ lfsr_q[3] ^ lfsr_q[4] ^ lfsr_q[7] ^ data_in[1] ^ data_in[4] ^ data_in[5] ^ data_in[7] ^ data_in[10] ^ data_in[11] ^ data_in[12] ^ data_in[15];
    lfsr_c[2] = lfsr_q[0] ^ lfsr_q[3] ^ lfsr_q[4] ^ lfsr_q[5] ^ data_in[2] ^ data_in[5] ^ data_in[6] ^ data_in[8] ^ data_in[11] ^ data_in[12] ^ data_in[13];
    lfsr_c[3] = lfsr_q[1] ^ lfsr_q[4] ^ lfsr_q[5] ^ lfsr_q[6] ^ data_in[3] ^ data_in[6] ^ data_in[7] ^ data_in[9] ^ data_in[12] ^ data_in[13] ^ data_in[14];
    lfsr_c[4] = lfsr_q[0] ^ lfsr_q[1] ^ lfsr_q[3] ^ lfsr_q[5] ^ data_in[0] ^ data_in[3] ^ data_in[6] ^ data_in[7] ^ data_in[8] ^ data_in[9] ^ data_in[11] ^ data_in[13];
    lfsr_c[5] = lfsr_q[0] ^ lfsr_q[3] ^ lfsr_q[4] ^ lfsr_q[7] ^ data_in[0] ^ data_in[1] ^ data_in[3] ^ data_in[6] ^ data_in[7] ^ data_in[8] ^ data_in[11] ^ data_in[12] ^ data_in[15];
    lfsr_c[6] = lfsr_q[0] ^ lfsr_q[1] ^ lfsr_q[4] ^ lfsr_q[5] ^ data_in[1] ^ data_in[2] ^ data_in[4] ^ data_in[7] ^ data_in[8] ^ data_in[9] ^ data_in[12] ^ data_in[13];
    lfsr_c[7] = lfsr_q[0] ^ lfsr_q[1] ^ lfsr_q[2] ^ lfsr_q[5] ^ lfsr_q[6] ^ data_in[2] ^ data_in[3] ^ data_in[5] ^ data_in[8] ^ data_in[9] ^ data_in[10] ^ data_in[13] ^ data_in[14];

  end 

  always @(posedge clk)
  begin
    if(rst) lfsr_q <= {8{1'b0}};
		else lfsr_q <= crc_en ? lfsr_c : lfsr_q;
  end 
endmodule 
