`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:10:13 03/12/2015 
// Design Name: 
// Module Name:    i2c_setup_rom 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////



module i2c_setup_rom #(parameter SI7020_ADDR = 8'h80, SI1145_ADDR = 8'hC0,          // SI7020 and SI1145 slave addresses, shifted left one bit with '0' LSB

                     // SI7020 I2C commands from Table 11, p. 19 of SI7020 UG
                     MSR_RH_HOLD = 8'hE5, MSR_RH_NHOLD =	8'hF5, MSR_TEMP_HOLD	= 8'hE3, MSR_TEMP_NHOLD = 8'hF3, R_TEMP_PREV = 8'hE0, RST = 8'hFE,
					 WR_USR1 = 8'hE6, RD_USR1 = 8'hE7, RD_ID_1 = 8'hFA, RD_ID_2 = 8'hFC, RD_FRMWR = 8'h84, 
						
					 // SI1145 I2C parameters from CH4 of SI1145 UG
					 PART_ID = 8'h00, REV_ID = 8'h01, INT_CFG = 8'h03, IRQ_ENABLE = 8'h04, PSLED12_SELECT = 8'h02, PSLED3_SELECT = 8'h03, HW_KEY = 8'h07, PS_LED21 = 8'h0F, PS_LED3 = 8'h10,
					 PARAM_WR = 8'h17, CMD = 8'h18, RESPONSE = 8'h20, INT_STATUS = 8'h21, ALS_VIS_DATA0 = 8'h22, ALS_VIS_DATA1 = 8'h23, ALS_IR_DATA0 = 8'h24, ALS_IR_DATA1 = 8'h25,
					 PS1_DATA0 = 8'h26, PS1_DATA1 = 8'h27, PS2_DATA0 = 8'h28, PS2_DATA1 = 8'h29, PS3_DATA0 = 8'h2A, PS3_DATA1 = 8'h2B,
					 PARAM_RD = 8'h2E, CHIP_STAT = 8'h30, UCOEF0 = 8'h13, UCOEF1 = 8'h14, UCOEF2 = 8'h15, UCOEF3 = 8'h16, AUX_DATA0 = 8'h2C, AUX_DATA1 = 8'h2D)
						
					(input [7:0] addr, output reg [7:0] slave_addr, output reg [7:0] reg_addr, output reg [7:0] data, output reg WR, output reg RD, output reg retry, output reg [1:0] rd_bytes,
					output reg no_data);
	 
	 always @(addr)
        case (addr)
		  
//                                                                             					I2C slave addr     		byte addr        data (if req'd)   write   read  retry     # of read
//                                                                               					(8 bits)         		(8 bits)           (8 bits)                  (y/n)      bytes    	no data				  
			  						
			  8'h00		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI7020_ADDR,			RD_USR1, 				8'h00, 		1'b0, 	1'b1, 1'b1, 	2'b00, 		1'b0};					// 																							  
		  
		      8'h01		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI7020_ADDR,			MSR_TEMP_NHOLD,     	8'h00, 		1'b0, 	1'b1, 1'b1, 	2'b10, 		1'b0};					// 			
			  8'h02		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI7020_ADDR,			MSR_TEMP_NHOLD,        	8'h00, 		1'b0, 	1'b1, 1'b1, 	2'b10, 		1'b0};					//
			  
			  8'h03		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI7020_ADDR,			MSR_TEMP_HOLD,          8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b10, 		1'b0};					//
			  8'h04		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI7020_ADDR,			MSR_TEMP_HOLD,          8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b10, 		1'b0};
			  
			  // host must write 0x17 to the HW_KEY register for proper operation	
			  8'h09		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			HW_KEY, 				8'h17, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};					//
			  
			  // read back the PART_ID (0x45 for Si1145) and the REV_ID (0x00) to check that part is correct
			  8'h0A		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PART_ID, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  8'h0B		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			REV_ID, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};

			  // write the value 0xB7 into the PARAM_WR register  0xB7 corresponds to all measurements except aux channel set in CHLIST parameter RAM location
			  8'h0C		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PARAM_WR, 				8'hB7, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  // [7] = EN_UV, [6] = EN_AUX, [5] = EN_ALS_IR, [4] = EN_ALS_VIS, [3] unused, [2] = EN_PS3, [1] = EN_PS2, [0] = EN_PS1
			  
			  // write the value 0xA1 into the CMD register  0xA1 will write the value in the PARAM_WR location into offset 0x01 (CHLIST)
			  8'h0D		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CMD, 					8'hA1, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  // read back the value placed into CHLIST by the write to CMD above
			  8'h0E		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PARAM_RD, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  
			  // write the PS LED current values (irLED current sunk during a measurment) set to maximum current for LED1, LED2 and 3 set no current
			  8'h0F		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS_LED21, 				8'h0F, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  
			  // write the PS LED current values (irLED current sunk during a measurment) set to minimum current for LED1, LED2 and 3 set no current
              //8'h0F        :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,            PS_LED21,                 8'h01,         1'b0,     1'b1, 1'b0,     2'b00,         1'b0};
                            	  
			  8'h10		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS_LED3, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  
			  // set INT_OE bit (interrupt output enable)
			  8'h11		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			INT_CFG, 				8'h01, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  // set interrupt enables for PS3/2/1 measurments
			  8'h12		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			IRQ_ENABLE,    			8'h1C, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  // [7:5] = unused, [4] = PS3_IE, [3] = PS2_IE, [2] = PS1_IE, [1] = unused, [1] = ALS_IE
			  
			  // specify the LED pin driven during PS1, PS2, PS3 measurements
			  // write the value 0x11 into the PARAM_WR register  0x11 corresponds to driving LED1 for both PS2 and PS1 measurement
			  8'h13		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PARAM_WR, 				8'h11, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  // [7] = unused, [6:4] = xx1 PS2 LED1 drive enabled, [3] = unused, [2:1] = xx1 PS1 LED1 drive enabled 
			  
			  // write the value 0xA2 into the CMD register  0xA2 will write the value in the PARAM_WR location into offset 0x02 (PSLED12_SELECT)
              8'h14     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        CMD,                    8'hA2,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};
			  
              // write the value 0x01 into the PARAM_WR register  0x01 corresponds to driving LED1 for both PS3
              8'h15     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        PARAM_WR,               8'h01,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};
              // [7:4] = unused, [3] = unused, [2:1] = xx1 PS3 LED1 drive enabled 			  
			  
			  // write the value 0xA3 into the CMD register  0xA3 will write the value in the PARAM_WR location into offset 0x03 (PSLED3_SELECT)
              8'h16     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        CMD,                    8'hA3,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};              
                           
              // write the value 0x01 into the PARAM_WR register  0x01 corresponds to ADC clock divided by 4
              8'h17     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        PARAM_WR,               8'h02,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};
              // [7:3] = unused, [2:0] = PS_ADC_GAIN               
                           
              // write the value 0xAB into the CMD register  0xAB will write the value in the PARAM_WR location into offset 0x0B (PS_ADC_GAIN)
              8'h18     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        CMD,                    8'hAB,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};              
                           
			  // write the default UV coefficients into the UCOEF registers (0x7B, 0x6B, 0x01, 0x00)
			  8'h19		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			UCOEF0, 				8'h7B, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  8'h1A		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			UCOEF1, 				8'h6B, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  8'h1B		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			UCOEF2, 				8'h01, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  8'h1C		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			UCOEF3, 				8'h00, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  
			  // load the calibration parameters for the specific part into registers 0x22 - 0x2D
			  8'h1D		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CMD, 					8'h12, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};			  
			  
			  // force a proximity sense and ambient light reading (PSALS_FORCE = 0x07)
			  8'h1E		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CMD, 					8'h07, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};

			  // read the chip status register to see that the device is carrying out the measurement
			  8'h1F		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CHIP_STAT,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  // read the chip status register to see that the device is carrying out the measurement
			  8'h20		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CHIP_STAT,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  // read the chip status register to see that the device is carrying out the measurement
			  8'h21		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CHIP_STAT,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  // read the chip status register to see that the device is carrying out the measurement
			  8'h22		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CHIP_STAT,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
		  
			  // clear the interrupt status register at the end of reading results.  Start of the results read here *****************************************************************************************************************  

			  // read the data registers for the measurement values
			  8'h40		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			ALS_VIS_DATA0,			8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //CE
			  8'h41		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			ALS_VIS_DATA1,			8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01
			  
			  8'h42		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			ALS_IR_DATA0,			8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //63			  
			  8'h43		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			ALS_IR_DATA1,			8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01
			  
			  8'h44		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS1_DATA0,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //03
			  8'h45		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS1_DATA1,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01
			  
			  8'h46		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS2_DATA0,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //04
			  8'h47		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS2_DATA1,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01
			  
			  8'h48		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS3_DATA0,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01
			  8'h49		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			PS3_DATA1,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //01


			  
			  // read the UV index data
			  8'h4A		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			AUX_DATA0, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0}; //0D
			  8'h4B		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			AUX_DATA1, 				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  
              // clear the interrupts
              8'h4C     :        {slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,        INT_STATUS,             8'h1C,      1'b1,   1'b0, 1'b0,     2'b00,      1'b0};
              
              
              
			  
			  // clear the response register with a NOOP to the CMD register
			  8'h4D		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			CMD, 					8'h00, 		1'b1, 	1'b0, 1'b0, 	2'b00, 		1'b0};
			  
			  // read the response register
			  8'h4E		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {SI1145_ADDR,			RESPONSE,				8'h00, 		1'b0, 	1'b1, 1'b0, 	2'b00, 		1'b0};
			  
			 // 8'h0C		:		{slave_addr, reg_addr, data, WR, RD, retry, rd_bytes, no_data} <= {RST, RST, 1'b1, 1'b0, 1'b0, 2'b00, 1'b0};	
			  
          default		:  	begin
							slave_addr <= 8'h00;
							reg_addr <= 8'h00;
							data <= 8'h00;		// 0x17 is the req'd HW_KEY value for the SI1145, 0x06 is the software reset command for broadcast 0x00 I2C address
							WR <= 1'b0;
							RD <= 1'b0;
							retry <= 1'b0;
							rd_bytes <= 2'b00;
							no_data <= 1'b0;
							end
         endcase



endmodule
