`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    11:16:07 02/12/2017 
// Design Name: 
// Module Name:    top 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module top #(parameter RAM_ADDR_BITS = 7) (input raw_clk_p, input rstn,
				// I2C interface
				inout SDA, inout SCL, 
				// User / PMOD signals
				input I2C_RST, input INIT, input WR, input RD, input SELECT_1145, output data_out_led, output ack_or_complete, (* MARK_DEBUG = "true" *) input int,
				// DCM signal
				output locked, output clk_slow_test);

reg [63:0] debounce_switch, debounce_rst, rst_delayed, debounce_WR, debounce_RD;
reg [127:0] debounce_I2C_RST, debounce_INIT;
reg [1:0] debounce_i2c_rst, debounce_init;
wire debounce_wr, debounce_rd;
wire clk_i, clk_slow, switch_debounced, rst_debounced, valid, complete, ack, lockedn;
wire [31:0] data_out;
reg [19:0] live_counter;
wire [4:0] STATE_view;
wire [7:0] slave_addr, reg_addr, reg_addr_init, data_in, init_data;
wire [7:0] slave_addr_f, reg_addr_f, data_in_f;			
reg [7:0] rom_addr;
reg i2c_rst, wr, rd, pulse_i2c_rst, pulse_init;	
wire WR_rom, RD_rom, retry, no_data, no_data_f;
wire [1:0] rd_bytes;
(* FSM_ENCODING="USER" *) reg [2:0] STATE;

localparam	SI7020_ADDR		=	8'h80;			// APPLIES TO PMOD, note device address is 7'h40, shifted left with a '0' LSB indicates write
localparam  SI1145_ADDR		=	8'hC0;			// APPLIES TO PMOD, note device address is 7'h60, shifted left with a '0' LSB indicates write

localparam	MSR_RH_HOLD		=	8'hE5;
localparam	MSR_RH_NHOLD	=	8'hF5;
localparam	MSR_TEMP_HOLD	=	8'hE3;
localparam	MSR_TEMP_NHOLD	=	8'hF3;
localparam	R_TEMP_PREV		=	8'hE0;
localparam	RST				=	8'hFE;
localparam	WR_USR1			=	8'hE6;
localparam	RD_USR1			=	8'hE7;
localparam	RD_ID_1			=	8'hFA;
localparam	RD_ID_2			=	8'hFC;
localparam	RD_FRMWR		=	8'h84;

localparam  PART_ID 		= 	8'h00;
localparam	REV_ID 			= 	8'h01;
localparam INT_CFG          =   8'h03;
localparam IRQ_ENABLE       =   8'h04;
localparam	PSLED12_SELECT  = 	8'h02;
localparam	PSLED3_SELECT 	= 	8'h03;
localparam	HW_KEY 			= 	8'h07;
localparam	PS_LED21 		= 	8'h0F;
localparam	PS_LED3 		= 	8'h10;
localparam	PARAM_WR 		= 	8'h17;
localparam	CMD 			= 	8'h18;
localparam	RESPONSE		=	8'h20;
localparam  INT_STATUS      =   8'h21;
localparam	ALS_VIS_DATA0 	= 	8'h22;
localparam	ALS_VIS_DATA1 	= 	8'h23;
localparam	ALS_IR_DATA0 	= 	8'h24;
localparam	ALS_IR_DATA1 	= 	8'h25;
localparam	PS1_DATA0 		= 	8'h26;
localparam	PS1_DATA1 		= 	8'h27;
localparam	PS2_DATA0 		= 	8'h28;
localparam	PS2_DATA1 		= 	8'h29;
localparam	PS3_DATA0 		= 	8'h2A;
localparam	PS3_DATA1 		= 	8'h2B;
localparam	PARAM_RD 		= 	8'h2E;
localparam  CHIP_STAT		=	8'h30;
localparam	UCOEF0 			= 	8'h13;
localparam	UCOEF1 			= 	8'h14;
localparam	UCOEF2 			= 	8'h15;
localparam	UCOEF3 			= 	8'h16;
localparam	AUX_DATA0 		= 	8'h2C;
localparam	AUX_DATA1 		= 	8'h2D;

localparam	SETUP_7020_START		=	8'h00;				// This is where the 7020 I2C commands start in the i2c_setup_rom addresses
localparam	SETUP_7020_END			=	8'h04;				

localparam	SETUP_1145_START		=	8'h09;				// This is where the 1145 I2C commands start in the i2c_setup_rom addresses
localparam	SETUP_1145_END			=	8'h22;

localparam  SI1145_READ             =   8'h40;              // This is where the 1145 read results commands begin in the i2c_setup_rom addresses
localparam  SI1145_READ_END         =   8'h4C;

localparam IDLE 				    =   3'h0;
localparam RUN					    =   3'h1;
localparam ADDR				        =   3'h2;
localparam SETUP 				    =   3'h3;
localparam WAIT_SETUP		        =   3'h4;

localparam RETRY_CNT			    = 10'h3FF;

localparam GENERAL_CALL = 8'h00;

IBUFG clk_ibufg (.I(raw_clk_p), .O(clk_i));				// the raw 100 MHz clock is brought in on IBUFG, sent to MMCM

assign ack_or_complete = ack | complete;

always@(posedge clk_i) debounce_rst <= {debounce_rst [62:0], !rstn};
assign rst_debounced = &debounce_rst;                   // debounce the reset, provide to logic
always@(posedge clk_i) rst_delayed <= {rst_delayed [62:0], rst_debounced};  // delay the reset to the MMCM.  It will after and be released after the logic reset
assign locked = !lockedn;                               // invert the locked signal from MMCM, send to LED.  RED2 indicates MMCM unlocked

clock_module clk_box (.raw_clk(clk_i), /*.clk_fast(),*/ .clk_slow(clk_slow), .rst(rst_delayed[63]), .locked(lockedn));
// raw_clk input at 100MHz for ARTY board

always@(posedge clk_slow)
begin
debounce_WR <= {debounce_WR [62:0], WR};
debounce_RD <= {debounce_RD [62:0], RD};
debounce_I2C_RST <= {debounce_I2C_RST [126:0], I2C_RST};
debounce_INIT <= {debounce_INIT [126:0], INIT};
debounce_init <= {debounce_init[0], &debounce_INIT};
debounce_i2c_rst <= {debounce_i2c_rst[0], &debounce_I2C_RST};
pulse_i2c_rst <= (debounce_i2c_rst == 2'b01)? 1'b1 : 1'b0;
pulse_init <= (debounce_init == 2'b01)? 1'b1 : 1'b0;
live_counter <= live_counter + 1'b1;
end

assign debounce_wr = &debounce_WR;
assign debounce_rd = &debounce_RD;
assign clk_slow_test = live_counter [19];


i2c_interface	#(RETRY_CNT) interface (.clk_slow(clk_slow), .slave_addr(slave_addr_f), .reg_addr(reg_addr_f), .data_in(data_in_f),
									.WR_fast(debounce_wr | wr), .RD_fast(debounce_rd | rd), .SDA(SDA), .SCL(SCL), .data_out(data_out), .valid(valid),
									.complete(complete), .rst(rst_debounced), .retry(retry), .rd_bytes(rd_bytes), .no_data(no_data_f), .ack(ack));


always@(posedge clk_slow)
begin

	if (!rstn) STATE <= IDLE;

	else
		begin
		case (STATE)
				
				IDLE		:	begin
								if (pulse_init)     // If user presses the INIT pushbutton, run the initialization plus some extra commands
								begin
								STATE <= SETUP;								
								rom_addr <= (SELECT_1145)? SETUP_1145_START : SETUP_7020_START;     // decide where to enter the command ROM based on the DIP switch
								end
								else if (pulse_i2c_rst)     // If user presses the I2C_RST button, send the global address software reset command
									begin
									STATE <= RUN;
									i2c_rst <= 1'b1;
									wr <= 1'b1;
									end
									else if (!int)
									   begin
									   STATE <= SETUP;								
                                       rom_addr <= SI1145_READ;     // enter SI1145 commands to read the results
									   end
									   else
                                           begin
                                           i2c_rst <= 1'b0;
                                           wr <= 1'b0;
                                           end
								end
								
				RUN		:	begin
								if (complete) STATE <= IDLE;
									else wr <= 1'b0;
								end
								
				SETUP		:	begin
								wr <= WR_rom;
								rd <= RD_rom; 
								STATE <= WAIT_SETUP;
								end
										
				WAIT_SETUP	:	begin
									wr <= 1'b0;
									rd <= 1'b0;
				
									if (complete)
										begin
										if ((rom_addr < SETUP_7020_END) | ((SELECT_1145) && (rom_addr < SETUP_1145_END)) | (!int && (rom_addr < SI1145_READ_END)))
											begin
											rom_addr <= rom_addr + 1'b1;
											STATE <= SETUP;
											end
											else
												begin
												rom_addr <= (SELECT_1145) ? SETUP_1145_START : SETUP_7020_START;
												STATE <= IDLE;
												end
										end
									end	
		
			default		:	begin
								STATE <= IDLE;
								end
								
			endcase
		
		end
end

assign slave_addr_f = (i2c_rst)? GENERAL_CALL : slave_addr;
assign reg_addr_f = (i2c_rst)? 8'h06 : reg_addr_init;	// 8'h06 = software reset
assign data_in_f = (i2c_rst)? 8'h06 : init_data;
assign no_data_f = (i2c_rst)? 1'b1 : no_data;

assign data_out_led = ^{data_out, valid};


i2c_setup_rom #(SI7020_ADDR, SI1145_ADDR, MSR_RH_HOLD, MSR_RH_NHOLD, MSR_TEMP_HOLD, MSR_TEMP_NHOLD, R_TEMP_PREV, RST, WR_USR1, RD_USR1, RD_ID_1, RD_ID_2, RD_FRMWR,
				PART_ID, REV_ID, INT_CFG, IRQ_ENABLE, PSLED12_SELECT, PSLED3_SELECT, HW_KEY, PS_LED21, PS_LED3, PARAM_WR, CMD, RESPONSE, INT_STATUS, 
				ALS_VIS_DATA0, ALS_VIS_DATA1, ALS_IR_DATA0, ALS_IR_DATA1, PS1_DATA0, PS1_DATA1, PS2_DATA0, PS2_DATA1, PS3_DATA0, PS3_DATA1, PARAM_RD, CHIP_STAT, UCOEF0, UCOEF1, UCOEF2, UCOEF3, AUX_DATA0, AUX_DATA1) 
				PMOD_initialization (.addr(rom_addr), .slave_addr(slave_addr), .reg_addr(reg_addr_init), .data(init_data), .WR(WR_rom), .RD(RD_rom), .retry(retry), .rd_bytes(rd_bytes),
									 .no_data(no_data));




endmodule
